part of katana_router_builder;

class RouterGenerator extends GeneratorForAnnotation<AppRoute> {
  RouterGenerator();

  static const _typeChecker = TypeChecker.fromRuntime(PagePath);

  @override
  FutureOr<String> generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    BuildStep buildStep,
  ) async {
    if (!element.library!.isNonNullableByDefault) {
      throw InvalidGenerationSourceError(
        "Generator cannot target libraries that have not been migrated to "
        "null-safety.",
        element: element,
      );
    }

    if (element is! TopLevelVariableElement) {
      throw InvalidGenerationSourceError(
        "`@AppRoute()` can only be used on top level variable field.",
        element: element,
      );
    }
    final assets = buildStep.findAssets(Glob("**.dart"));
    await for (final asset in assets) {
      final lib = LibraryReader(
        await buildStep.resolver.libraryFor(
          asset,
          allowSyntaxErrors: false,
        ),
      );
      for (final annotatedElement in lib.annotatedWith(_typeChecker)) {
        final element = annotatedElement.element;
        if (element is! ClassElement) {
          continue;
        }
        element.fields.forEach((e) {
          print(e);
          print(e.computeConstantValue());
          print(e.isStatic);
          print(e.isConst);
        });
      }
    }

    final generated = Library(
      (l) => l
        ..body.addAll(
          [],
        ),
    );
    final emitter = DartEmitter();
    return DartFormatter().format(
      "${generated.accept(emitter)}",
    );
  }
}
